import * as CP from '@scheduleopt/optalcp';

// The input is la17:
let input = [
  10, 10,
  4, 18, 7, 21, 9, 41, 2, 45, 3, 38, 8, 50, 5, 84, 6, 29, 1, 23, 0, 82,
  8, 57, 5, 16, 1, 52, 7, 74, 2, 38, 3, 54, 6, 62, 9, 37, 4, 54, 0, 52,
  2, 30, 4, 79, 3, 68, 1, 61, 8, 11, 6, 89, 7, 89, 0, 81, 9, 81, 5, 57,
  0, 91, 8, 8, 3, 33, 7, 55, 5, 20, 2, 20, 4, 32, 6, 84, 1, 66, 9, 24,
  9, 40, 0, 7, 4, 19, 8, 7, 6, 83, 2, 64, 5, 56, 3, 54, 7, 8, 1, 39,
  3, 91, 2, 64, 5, 40, 0, 63, 7, 98, 4, 74, 8, 61, 1, 6, 6, 42, 9, 15,
  1, 80, 7, 39, 8, 24, 3, 75, 4, 75, 5, 6, 6, 44, 0, 26, 2, 87, 9, 22,
  1, 15, 7, 43, 2, 20, 0, 12, 8, 26, 6, 61, 3, 79, 9, 22, 5, 8, 4, 80,
  2, 62, 3, 96, 4, 22, 9, 5, 0, 63, 6, 33, 7, 10, 8, 18, 1, 36, 5, 40,
  1, 96, 0, 89, 5, 64, 3, 95, 9, 23, 7, 18, 8, 15, 2, 64, 6, 38, 4, 8
];

let m = new CP.Model();
const nbJobs = input.shift();
const nbMachines = input.shift();

// 2D array of interval variables: machines[machineId] are all the operations on
// that machine. Initially empty.
let machines = [];
for (let j = 0; j < nbMachines; j++)
  machines[j] = [];

// For each job we keep an expressions that represents its end time:
let ends = [];

for (let i = 0; i < nbJobs; i++) {
  let prev = undefined;
  // Each job visits all machines:
  for (let j = 0; j < nbMachines; j++) {
    const machineId = input.shift();
    const duration = input.shift();
    let operation = m.intervalVar({length: duration, name: `J${i}O${j}M${machineId}`});
    machines[machineId].push(operation);
    // Operations in the job are performed one after the other:
    if (prev !== undefined)
      prev.endBeforeStart(operation);
    prev = operation;
  }
  // The end of the job is the end of the last operation:
  ends.push(prev.end());
}

// Operations on the same machine cannot overlap:
for (let j = 0; j < nbMachines; j++)
  m.noOverlap(machines[j]);

// Minimize the makespan
m.max(ends).minimize();

// First worker will run FDS with maximum propagation. The remaining workers
// (depending on the CPU core count) will run the default (LNS):
let params = {
  solverPath: "../bin/optalcp",
  workers: [{searchType: "FDS", noOverlapPropagationLevel: 4}],
  timeLimit: 60
};
// Allow to modify the parameters from the command line:
CP.parseParameters(params, "Usage: node jobshop-la17.js");

let result = await CP.solve(m, params);

if (result.error)
  console.log("An error ocurred");
else if (result.bestSolution) {
  for (let i = 0; i < nbJobs; i++) {
    let startTimes = [];
    for (let j = 0; j < nbMachines; j++)
      startTimes.push(result.bestSolution.getStart(machines[j][i]));
    console.log(`Start times for job ${i}: ${startTimes.join(" ")}`);
  }
}